/* ****************************************************** */
/*     jikkyo-robot program  ver.1  pic16f88              */
/*                                                        */
/*     program is CCS-C PICC Compiler                     */
/*                  (PCB & PCM Version 3.183)             */
/*                                                        */
/*     Y/M/D  2010/6/20 programmed by jikko company       */
/* ****************************************************** */
// ************************** I/O *******************************
//  port_a -------- input port
//    PA0 ----- infra sensa RIGHT & RS232C & main infra sensa
//    PA1 ----- infra sensa LEFT
//    PA2 ----- SWICH-3(SW3)
//    PA3 ----- SWICH-2(SW2)
//    PA4 ----- SWICH-1(SW1)
//    PA5 ----- SWICH-0(SW0)        mode0 mode1 mode2 mode3
//    PA6 ----- DIP-SWICH-0     PA6   1      1     0     0
//    PA7 ----- DIP-SWICH-1     PA7   1      0     1     0
//
// port_b ---------- output port
//                               LED            
//    PB0 ----- LED3                  LED3 LED2 LED1 LED0
//    PB1 ----- LED2                  PB0  PB1  PB2  PB3
//    PB2 ----- LED1
//    PB3 ----- LED0 & infra LED
//                                   stop    cw     ccw   stop
//    PB4 ----- MOTOR control RIGHT    0      1      0     1
//    PB5 ----- MOTOR control RIGHT    0      0      1     1
//    PB6 ----- MOTOR control LEFT     0      1      0     1
//    PB7 ----- MOTOR control LEFT     0      0      1     1
//
// ***************************************************************
#include <16f88.H>                     // include file set
#fuses INTRC_IO,NOWDT,PUT,NOPROTECT,NOLVP,NOMCLR,BROWNOUT
#fuses NOCPD,NOWRT,NODEBUG,CCPB0

#use Delay(Clock=4000000)	            // cpu speed 4MHz set (intosc)

#byte port_a=5		      // port_a address set bank0 05h
#byte port_b=6		      // port_b address set bank0 06h

#define int_per_sec 61	// 4000000/(4*256*64)=61.035
//                         interrupt cycle = 1.0*256*64=16.384msec

int int_count;		// interrupt times count
int infra;        // infra data
int qs_mode;      // Q-STEER mode=0 A or B   mode=1 C or D
int led_moni_on,led_moni_off;          // led monita on & off

//   ԍ     0    1    2    3    4    5    6    7      8    9
//           stop    E Oi E `  E
int move_led[10]={0x00,0x04,0x02,0x06,0x01,0x05,0x03,0x07,0x00,0x00};
int move_run[10]={0x00,0x10,0xa0,0x90,0x40,0x50,0x60,0xf0,0x20,0x80};

//   ********      *********
//   ԍ@@   PB7 PB6 PB5 PB4  run data  PB0  PB1  PB2  PB3  led data    3LED monitor
//      0      stop      0   0   0   0     0x00     0    0    0    -      0x00    @@
//      1            0   0   0   1     0x10     0    0    1    -      0x04    @@
//      2            1   0   1   0     0xa0     0    1    0    -      0x02    @@
//      3          1   0   0   1     0x90     0    1    1    -      0x06        
//      4      E      0   1   0   0     0x40     1    0    0    -      0x01        
//      5      Oi      0   1   0   1     0x50     1    0    1    -      0x05        
//      6      E    0   1   1   0     0x60     1    1    0    -      0x03        
//      7      `    1   1   1   1     0xf0     1    1    1    -      0x07        
//      8      ށ@@0   0   1   0     0x20     0    0    0    -      0x00    ~  ~  ~ not LED
//      9      Eށ@@1   0   0   0     0x80     0    0    0    -      0x00    ~  ~  ~ not LED
//   @``Ƃ́A܂ƉE܂0.3msecJԂđOi铮
//
//
//   Ԑݒ       0    1    2    3    4    5    6    7
int move_time[8]={0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07};
int times_led[8]={0x08,0x0c,0x0a,0x0e,0x09,0x0d,0x0b,0x0f};

//   ********      *********
//   Ԕԍ@@ԁ@   data      PB0  PB1  PB2  PB3   led data    LED
//      0      0   0  sec   0x00       0    0    0    1      0x08      @@  
//      1      1   0.5sec   0x01       0    0    1    1      0x0c      @@  
//      2      2   1  sec   0x02       0    1    0    1      0x0a      @@  
//      3      3   1.5sec   0x03       0    1    1    1      0x0e            
//      4      4   2  sec   0x04       1    0    0    1      0x09            
//      5      5   2.5sec   0x05       1    0    1    1      0x0d            
//      6      6   3  sec   0x06       1    1    0    1      0x0b            
//      7      7   3.5sec   0x07       1    1    1    1      0x0f            

//       eep-rom data seting dimension(MAX=64)
int ee_rom_data[64];

//       mode3 key data
int ir_data1,ir_data2,ir_rund,ir_mode;

// eep-rom data  adress 0x2100
#rom 0x2100={0x07,0x06, 0x00,0x00, //  1 cyorocyoro 6time  ,  2 stop
             0x00,0x00, 0x00,0x00, //  3 stop              ,  4 stop
             0x00,0x00, 0x00,0x00, //  5 stop              ,  6 stop
             0x00,0x00, 0x00,0x00, //  7 stop              ,  8 stop
             0x00,0x00, 0x00,0x00, //  9 stop              ,  10 stop
             0x00,0x00, 0x00,0x00, // 11 stop              ,  12 stop
             0x00,0x00, 0x00,0x00, // 13 stop              ,  14 stop
             0x00,0x00, 0x00,0x00, // 15 stop              ,  16 stop
             0x00,0x00, 0x00,0x00, // 17 stop              ,  18 stop
             0x00,0x00, 0x00,0x00, // 19 stop              ,  20 stop
             0x00,0x00, 0x00,0x00, // 21 stop              ,  22 stop
             0x00,0x00, 0x00,0x00, // 23 stop              ,  24 stop
             0x00,0x00, 0x00,0x00, // 25 stop              ,  26 stop
             0x00,0x00, 0x00,0x00, // 27 stop              ,  28 stop
             0x00,0x00, 0x00,0x00, // 29 stop              ,  30 stop
             0x00,0x00, 0x00,0x00} // 31 stop              ,  32 stop

/* ++++++++++++++ interrupt set +++++++++++++++++ */
// +++++++++++++++++++++++++++++++++++++++++++++++++
//  Timer0 is setuped below.
//    clock : 4MHz then cycle : 1.0usec
//    prescale : 61
//      then interrupt cycle = 1.0*256*64=16.384msec
//      then interval = 16.384msec
//    Include the standard header file.
// +++++++++++++++++++++++++++++++++++++++++++++++++
interrupt_set(){
	setup_counters(RTCC_INTERNAL,RTCC_DIV_64); // RTCC_INTERNAL=CLOCK USE 1/64
}

/* +++++++++ RTCC interrupt function ++++++++++++++ */
#INT_RTCC			          // interrupt tmr0
rtcc_isr(){			          // This function is called every time
	if(--int_count==0){	  	 // the RTCC(timer0) overflows(255-->0)
		int_count=int_per_sec;// 61times/second ( 16.384*61=0.99942sec = about 1sec)
	}
   if(int_count>=30){port_b=led_moni_on;}
   else port_b=led_moni_off;
}

/* +++++++++++++++ i/o set function ++++++++++++++ */
io_set(){
	set_tris_a(0xff);         // port_a = input seting
	set_tris_b(0x00);         // port_b = output seting
   port_b=0x00;              // motor stop
   led_moni_off=0x00;        // LED monitor off
}

/* ********************** stop **************** */
stop_clr(){                  // stop
      port_b = 0x00;         // stop & led clr
}
stop_brake(){                // stop brake
      port_b = 0xf0;         // brake
      delay_ms(100);         // 100msec brake
      port_b = 0x00;         // stop & led clr
}

/* ********************** cyorocyoro **************** */
cyoro(int time){                          // ``si֍ssj
   int i;

   for(i=0;i<time;i++){
      port_b = move_led[1] | 0xd0;              //  ( brake   )
      delay_ms(200);                            //  200msec run
      port_b = move_led[4] | 0x70;              //  E( @brake )
      delay_ms(200);                            //  200msec run
   }
   stop_clr();             //  stop
}

/* ********************* eep-rpm function  ************************* */
clr_data(){                // eep_rom_data[0] to [63] clr
   int i;
   for(i=0;i<=64;i++){
      ee_rom_data[i]=0x00;
   }
}

eep_write(){             // eep_rom_data[i] ====> eeprom  (writing program)
   int i;
   long rom_adress;

   rom_adress = 0x2100;             // rom_adress first data
   for(i=0;i<64;i++){
      write_eeprom(rom_adress,ee_rom_data[i]);   //rom data write
      rom_adress++;
   }
}

eep_read(){              // eeprom =====> ee_rom_data[i] (reading program)
   int i;
   long rom_adress;

   rom_adress = 0x2100;             // rom_adress first data
   for(i=0;i<64;i++){
      ee_rom_data[i]=read_eeprom(rom_adress);   //rom data read
      rom_adress++;
   }
}

eep_err(){                // eep err led
   led_moni_on = 0x08;              // LED0 ON
	enable_interrupts(INT_RTCC);		// interrupt
	enable_interrupts(GLOBAL);		   // interrupt start OK
   delay_ms(1000);                  // waite 1sec
   while(1){
   }
}

eep_end(){                 // end set
   led_moni_on = 0x01;              // LED3 ON
	enable_interrupts(INT_RTCC);		// interrupt
	enable_interrupts(GLOBAL);		   // interrupt start OK
   delay_ms(3000);                  // waite 3sec
   led_moni_on = 0x00;              // LED OFF
   disable_interrupts(GLOBAL);      // interrupt stop
}

run_eep(){                 // eep_rom_program data reading run
   int i,j;
   int time;

   for(i=0;i<32;i++){
      port_b = move_run[ee_rom_data[2*i]] | move_led[ee_rom_data[2*i]]; // read move data & led
      time = ee_rom_data[2*i+1];                                        // read time data
      if(time==0){goto exit_run_eep;}                                      // end chek
      if(ee_rom_data[2*i] ==0x07){cyoro(time);}                         // if  cyoro mode run
      else {for(j=0;j<time;j++){delay_ms(500);}}                        // else 0.5ms*time
   }
   exit_run_eep:
   stop_clr();       // port_b clr
   eep_end();        // end display set ( 3sec LED ) & return
}

/* ****************** mode-0 function ****************** */
/* @@@fvOP@Q@@@                      */
/* @@@ZT䎩[h                            */
/* ***************************************************** */
// ---------- run1 = demo1 -----------
run1(){
  int i;
	disable_interrupts(GLOBAL);		       // interrupt stop
   port_b = move_led[5] | move_run[5];     //  Oi
   delay_ms(2000);                         //  waite 2sec
   port_b = move_led[2] | move_run[2];     //  
   delay_ms(2000);                         //  waite 2sec
   port_b = move_led[3] | move_run[3];     //  
   delay_ms(2000);                         //  waite 2sec
   port_b = move_led[6] | move_run[6];     //  E
   delay_ms(2000);                         //  waite 2sec
   cyoro(5);                               //  cyorocyoro  5times
   stop_clr();                             //  stop
	enable_interrupts(GLOBAL);		          //  interrupt start
}
// ---------- run2 = demo2 ----------
run2(){
  int i;
   disable_interrupts(GLOBAL);		       // interrupt stop
   for(i=1;i<7;i++){
      port_b = move_led[i] | move_run[i];  //  run data output
      delay_ms(1500);                      //  waite 1.5sec
      stop_brake();                        //  brake
      delay_ms(1500);                      //  waite 1.5sec
   }
   cyoro(3);                               //  cyorocyoro  3times
   stop_clr();                             //  stop
	enable_interrupts(GLOBAL);		          // interrupt start
}
// ---------- run3 = jiritu program ----------
inf_check(){                     // sensor check program
   int i,sen_r,sen_l,sen;

   sen=0;
   sen_r=0;sen_l=0;

   set_pwm1_duty(0x1A);          // 38KHz on => [infra =on ]

   for(i=0;i<10;i++){            // sensor check 10 count
      if(input(PIN_A0)==0){sen_r++;} // right sensor check 10count
      if(input(PIN_A1)==0){sen_l++;} // left sensor check 10count
      delay_us(100);             // 100usec * 10count = 1mse about
   }

   if(sen_r>=8){sen=0x01;}       // right sensor [1]
   else{sen=0x00;}               // right sensor [0]
   if(sen_l>=8){sen=sen+0x02;}   // left sensor [1]
   else{sen=sen;}                // left sensor [0]

   set_pwm1_duty(0);             // PWM off

   delay_ms(1);                  // waite 1msec

   return(sen);                  // sensor data return
                                 // 0x00  off off ;0x03 on on  ;0x02 on off  ;0x01 off on
}

run3(int jiritu){                // sensor run program
   int sen;
	disable_interrupts(GLOBAL);		       // interrupt stop

   setup_ccp1(CCP_PWM);                // CCP1 PWM MODE
   setup_timer_2(T2_DIV_BY_1,0x1A,1);  // 38kHz set
   set_pwm1_duty(0);                   // PWM off

   while(1){
      sen=inf_check();

	   switch(sen){			   // sensor check
		case 0x00:                              // sensor off
         port_b = move_led[5] | move_run[5];  //  Oi
         delay_ms(100);
			break;

		case 0x01:                              // sensor off on
			if(jiritu==1){
               port_b = move_led[3] | move_run[3]; //  
               delay_ms(1000);                 		//  1sec
         }
			else{
               port_b = move_led[2] | move_run[2]; //  
               delay_ms(1000);                 		//  1sec
               port_b = move_led[3] | move_run[3]; //  
               delay_ms(1000);                 		//  1sec
         }
			break;

		case 0x02:                              // sensor on off
			if(jiritu==1){
               port_b = move_led[6] | move_run[6]; //  E
               delay_ms(1000);                 		//  1sec
         }
			else{
               port_b = move_led[2] | move_run[2]; //  
               delay_ms(1000);                 		//  1sec
               port_b = move_led[6] | move_run[6]; //  E
               delay_ms(1000);                 		//  1sec
         }
			break;

		case 0x03:                              // sensor on on
			port_b = move_led[2] | move_run[2];       //  
         delay_ms(1000);                 		      //  1sec
        	break;

		default:delay_ms(2);break;
    }
   }
}

mode0(){                // mode-0 main program
   int key;
   int jiritu;

   led_moni_on = 0x08;              // LED0 ON
	enable_interrupts(INT_RTCC);		// interrupt
	enable_interrupts(GLOBAL);		   // interrupt start OK

   while(1){
      key =~port_a & 0x3c;
   	switch(key){			                       // switch chek
		   case 0x04:jiritu=1;run3(jiritu);break;   // switch-3 sensor run2
		   case 0x08:jiritu=0;run3(jiritu);break;   // switch-2 sensor run1
		   case 0x10:run2();break;                  // switch-1 run3 demo1
		   case 0x20:run1();break;                  // switch-0 run4 demo2
  		default:delay_ms(20);break;
      }
	}
}

/* ****************** mode-1 function ****************** */
/* @@@蓮͎s[h @                       */
/* @@@i{bg̊wK[hj                        */
/* ***************************************************** */
mode1(){
   int i,key;
   long rom_adress;
   int rom_data;
   int move_d;

   led_moni_on = 0x04;              // LED2 ON
	enable_interrupts(INT_RTCC);		// interrupt
	enable_interrupts(GLOBAL);		   // interrupt start OK
   delay_ms(2900);                  // waite 3-sec

   clr_data();                      //ee_rom_data <====== 0 set

   i = 0;                      // read adress first data
   rom_data = 0;               // rom_data =0

   disable_interrupts(GLOBAL);   // interrupt stop
   led_moni_on = 0x00;              // LED OFF
   stop_clr();

   while(1){
      loop:
      key =~port_a & 0x3c;                // key chek
      if(key==0){goto loop;}              // no-key ---> loop
      delay_ms(20);                       // chataring waite 20msec
         while((~port_a) & 0x3c){}        // pull key waite
     	switch(key){			               // key  chek
		   case 0x04:eep_read();            // key-4 eep_rom reading
                   i=0;rom_data=0;
                   break;
		   case 0x08:rom_data++;            // key-3 data seting( 0 to 7 )
                   if(rom_data==8){rom_data=0;};
                   ee_rom_data[i]=rom_data;
                   break;
		   case 0x10:i++;                   // key-2 data set and next adress
                   rom_data=0;
                   break;
		   case 0x20:eep_write();           // key-1 eep_rom write & run
                   run_eep();
                   i=0;
                   rom_data=0;
                   break;
  		   default:delay_ms(2);break;       // default
      }
      if(i==31){eep_write();eep_err();}   // i max 31 err

      if((i%2)==0){port_b = move_led[ee_rom_data[i]];}   // move data led set
      if((i%2)!=0){port_b =times_led[ee_rom_data[i]];}   // time data led set
      delay_ms(10);                                      // waite 10ms
   }
}

/* ****************** mode-2 function ****************** */
/* @@@qrQRQbʐM[h @                       */
/* @@@|PRVAʐM[h                      */
/* ***************************************************** */

/* ++++++++++++ rs232 function ++++++++++++ */
/*                                          */
/*     RS232C receive format                */
/*       baud rate 1200 bps                 */
/*       data bit     8 bit                 */
/*       stop bit     1 bit                 */
/*       parity        none                 */
/*       end of line     CR                 */
/*       end of file     1A                 */
/*                                          */
/*  +++++++++++++++++++++++++++++++++++++++ */
hex_ascii(int data){
   int a;

   if(data>=0x40){a=data-0x40+10;}
   else{a=data-0x30;}
   return(a);
}
/* ------ 1word get function (ASCII CODE)------ */
rs_1word(){
	int rs_data,i;
   rs_data=0;
   i=0;
   while(~input(PIN_A0)){}
   delay_us(1000);
   while(1){
     if(input(PIN_A0)==0){rs_data=(rs_data>>1)+0x80;}
     else {rs_data=(rs_data>>1);}
     delay_us(830);      // 1bit time = 1000000/1200=833usec
     i++;
     if(i==8){goto exit_rs1word;}
   }
   exit_rs1word:
   return(rs_data);
}
/* ------------- data get function ----------- */

// --------------- rs232c receive program ----------
rs_receive(){
	int data1,data2,i,j;
   clr_data();
   data1=rs_1word();
   j=0;
   while(1){                              // data receive
     data1=rs_1word();
     data2=rs_1word();
     ee_rom_data[j]=hex_ascii(data1);
     ee_rom_data[j+1]=hex_ascii(data2);
     if((data1==0x30)&(data2==0x30)){goto exit_rs;}
     j=j+2;
     if(j==32){goto exit_rs;}
	}
   exit_rs:
   eep_write();                           // eep rom data writing

   for(i=0;i<64;i=i+2){
     if((ee_rom_data[i]==0)&(ee_rom_data[i+1]==0)){goto exit_rs_end;}
     port_b=move_led[ee_rom_data[i]];
     delay_ms(2000);
     port_b=times_led[ee_rom_data[i+1]];
     delay_ms(2000);
   }
   exit_rs_end:
   port_b=0x00;
   return(0);
}
// --------------- pokekon receive program -------------
poke_receive(){
	int data1,data2,i,j;
   clr_data();
   for(i=0;i<9;i++){
      data1=rs_1word();
   }
   j=0;
   while(1){                              // data receive
     data1=rs_1word();
     data2=rs_1word();
     ee_rom_data[j]=hex_ascii(data1);
     ee_rom_data[j+1]=hex_ascii(data2);
     if((data1==0x30)&(data2==0x30)){goto exit_poke;}
     j=j+2;
     if(j==32){goto exit_poke;}
	}
   exit_poke:
   eep_write();                           // eep rom data writing

   for(i=0;i<64;i=i+2){                   // resiv data led set
     if((ee_rom_data[i]==0)&(ee_rom_data[i+1]==0)){goto exit_poke_end;}
     port_b=move_led[ee_rom_data[i]];
     delay_ms(2000);
     port_b=times_led[ee_rom_data[i+1]];
     delay_ms(2000);
   }
   exit_poke_end:
   port_b=0x00;
   return(0);

}
// ------ run4 = rs232c receive -----
run4(){
	disable_interrupts(GLOBAL);		       // interrupt stop
   rs_receive();                           // rs232c get data
	enable_interrupts(GLOBAL);		          // interrupt start
   led_moni_on = 0x02;                     // LED3 ON
}
// ------- run5 = pokekon receive -----
run5(){
	disable_interrupts(GLOBAL);		       // interrupt stop
   poke_receive();                         // pokekon sio get data
	enable_interrupts(GLOBAL);		          // interrupt start
   led_moni_on = 0x02;                     // LED3 ON
}

mode2(){
   int key;

   led_moni_on = 0x02;              // LED3 ON

	enable_interrupts(INT_RTCC);	  	 // interrupt RTCC
	enable_interrupts(GLOBAL);		    // interrupt start OK
   delay_ms(3000);                   // waite 3sec
	disable_interrupts(INT_RTCC);	  	 // interrupt stop RTCC

   while(1){
      key =~port_a & 0x3c;
   	switch(key){			           // switch check
		   case 0x04:eep_read();break;  // sw-3 eep_rom reading
		   case 0x08:run5();break;      // sw-2 run5 pokekon rs232c reciv & eep-rpm write
		   case 0x10:run4();break;      // sw-1 run4 rs232c reciv & eep-rom write
		   case 0x20:                   // sw-0 run
               	disable_interrupts(GLOBAL);		       // interrupt stop
                  run_eep();
               	enable_interrupts(GLOBAL);		          // interrupt start
                  led_moni_on = 0x02;                     // LED3 ON
                  break;
     		default:delay_ms(20);break;
      }
	}
}

/* ****************** mode-3 function ****************** */
/* @@@erR䃂[h                        */
/* @@@@rN^[@erR                    */
/* @@@@\j[@@erR                    */
/* @@@@OH@@@erR                    */
/* @@@@o_C@p|rsddqR              */
/* ***************************************************** */
ir_run(){
   if(ir_rund==0x0b){
      cyoro(1);
   }
   else{
      port_b = move_led[ir_rund] | move_run[ir_rund]; //  f[^o
      delay_ms(200);
   }
   if((ir_mode==1)&(ir_rund==0x00)){stop_brake();}    // ir_mode=1 stop ==>brake
   if(ir_mode==0){stop_clr();}                        // ir_mode=0 clr
}

ir_qsrun(){
   port_b = move_led[ir_rund] | move_run[ir_rund]; //  f[^o
   delay_ms(50);
   if((ir_mode==1)&(ir_rund==0x00)){stop_brake();}    // ir_mode=1 stop ==>brake
   if(ir_mode==0){stop_clr();}                        // ir_mode=0 clr
}
//  +++++++++++++++++  victor mode  +++++++++++++++++
//       ch      ir_data1         ir_data2
//      1ch    0000 0011 0x03     0010 0001 0x21
//      2ch    0000 0011 0x03     0010 0010 0x22
//      3ch    0000 0011 0x03     0010 0011 0x23
//      4ch    0000 0011 0x03     0010 0100 0x24
//      5ch    0000 0011 0x03     0010 0101 0x25
//      6ch    0000 0011 0x03     0010 0110 0x26
//      7ch    0000 0011 0x03     0010 0111 0x27
//      8ch    0000 0011 0x03     0010 1000 0x28
//      9ch    0000 0011 0x03     0010 1001 0x29
//     10ch    0000 0011 0x03     0010 1010 0x2a
//     11ch    0000 0011 0x03     0010 1011 0x2b
//     12ch    0000 0011 0x03     0010 1100 0x2c
//   ++++++++++++++++++++++++++++++++++++++++++++++++++++
ir_rcv_vic(){                        // ir_receive victor function
   int a,i,j;
   int r_data;

   ir_next1:

   i=0;
   r_data=0;

   while(1){
         if(input(PIN_A0)==0){
            for(j=0;j<10;j++){delay_us(30);a=0;
               if(input(PIN_A0)!=0){goto noise1;}
            }
            while(~input(PIN_A0)){a=0;}
            while(1){
               if(input(PIN_A0)==1){a++;}
               else{goto next1;}
               delay_us(30);
            }
            next1:
            if(a>=30)r_data=(r_data>>1)+0x80;
            else r_data=(r_data>>1);
            i++;
         }
         noise1:
         if(i==8){ir_data1=r_data;
            if(ir_data1==0x03){r_data=0;}
            else {i=7;}
         }
         if(i==16){ir_data2=r_data;r_data=0;goto exit_rcv1;}
   }
   exit_rcv1:
   a = ir_data2 & 0x0f;
   	switch(a){			                            // switch chek
	     	case 0x01:ir_rund=0x01;break;              //  1ch  left front turn
	     	case 0x02:ir_rund=0x05;break;              //  2ch  front
	     	case 0x03:ir_rund=0x04;break;              //  3ch  right front turn
	     	case 0x04:ir_rund=0x03;break;              //  4ch  left turn
	     	case 0x05:ir_rund=0x00;break;              //  5ch  stop
	     	case 0x06:ir_rund=0x06;break;              //  6ch  right turn
	     	case 0x07:ir_rund=0x08;break;              //  7ch  left back
	     	case 0x08:ir_rund=0x02;break;              //  8ch  back
	     	case 0x09:ir_rund=0x09;break;              //  9ch  right back
	     	case 0x0a:ir_mode=0;ir_rund=0x00;break;    // 10ch  200msec mode
	     	case 0x0b:ir_rund=0x0b;break;              // 11ch  cyoro cyoro
	     	case 0x0c:ir_mode=1;ir_rund=0x00;break;    // 12ch  continue mode
   		default:ir_rund=0x00;delay_us(2);break;
      }
   ir_run();
   goto ir_next1;
}

//  +++++++++++++++++  sony mode  ++++++++++++++++++
//       ch      ir_data1         ir_data2
//      1ch    0000 0001 0x01     ----0001
//      2ch    0000 0011 0x03     ----0001
//      3ch    0000 0101 0x05     ----0001
//      4ch    0000 0111 0x07     ----0001
//      5ch    0000 1001 0x09     ----0001
//      6ch    0000 1011 0x0b     ----0001
//      7ch    0000 1101 0x0d     ----0001
//      8ch    0000 1111 0x0f     ----0001
//      9ch    0001 0001 0x11     ----0001
//     10ch    0001 0011 0x13     ----0001
//     11ch    0001 0101 0x15     ----0001
//     12ch    0001 0111 0x17     ----0001
//   ++++++++++++++++++++++++++++++++++++++++++++++++++++

ir_rcv_son(){                        // ir_receive sony function
   int a,i;
   int r_data;

   ir_next2:

   i=0;
   r_data=0;

   while(1){
         while(input(PIN_A0)){a=0;}
         while(1){
            if(input(PIN_A0)==0){a++;}
            else{goto next2;}
            delay_us(30);
         }
         next2:
         if(a>=25)r_data=(r_data>>1)+0x80;
         else r_data=(r_data>>1);
         i++;
         if(i==8){ir_data1=r_data;r_data=0;}
         if(i==12){r_data>>=4;ir_data2=r_data;r_data=0;goto exit_rcv2;}
   }
   exit_rcv2:
   a = ir_data1 & 0x1f;
   	switch(a){			                             // switch chek
	     	case 0x01:ir_rund=0x01;break;               //  1ch  left front turn
	     	case 0x03:ir_rund=0x05;break;               //  2ch  front
	     	case 0x05:ir_rund=0x04;break;               //  3ch  right front turn
	     	case 0x07:ir_rund=0x03;break;               //  4ch  left turn
	     	case 0x09:ir_rund=0x00;break;               //  5ch  stop
	     	case 0x0b:ir_rund=0x06;break;               //  6ch  right turn
	     	case 0x0d:ir_rund=0x08;break;               //  7ch  left back
	     	case 0x0f:ir_rund=0x02;break;               //  8ch  back
	     	case 0x11:ir_rund=0x09;break;               //  9ch  right back
	     	case 0x13:ir_mode=0;ir_rund=0x00;break;     // 10ch  200msec mode
	     	case 0x15:ir_rund=0x0b;break;               // 11ch  cyoro cyoro
	     	case 0x17:ir_mode=1;ir_rund=0x00;break;     // 12ch  continue mode
   		default:ir_rund=0x00;delay_us(2);break;
      }
   ir_run();
   goto ir_next2;
}

//  +++++++++++++++++  mitubish mode +++++++++++++++++
//       ch      ir_data1         ir_data2
//      1ch    0100 0111 0x47     0000 0000 0x00
//      2ch    0100 0111 0x47     0000 1000 0x08
//      3ch    0100 0111 0x47     0001 0000 0x10
//      4ch    0100 0111 0x47     0001 1000 0x18
//      5ch    0100 0111 0x47     0010 0000 0x20
//      6ch    0100 0111 0x47     0010 1000 0x28
//      7ch    0100 0111 0x47     0011 0000 0x30
//      8ch    0100 0111 0x47     0011 1000 0x38
//      9ch    0100 0111 0x47     0000 0001 0x01
//     10ch    0100 0111 0x47     0000 1001 0x09
//     11ch    0100 0111 0x47     0001 0001 0x11
//     12ch    0100 0111 0x47     0001 1001 0x19
//   ++++++++++++++++++++++++++++++++++++++++++++++++++++

ir_rcv_mit(){                        // ir_receive mitubisi function
   int a,i,j;
   int r_data;

   ir_next3:
   i=0;
   r_data=0;

   while(1){
         if(input(PIN_A0)==0){
            for(j=0;j<3;j++){delay_us(5);a=0;
               if(input(PIN_A0)!=0){goto noise3;}
            }
            while(~input(PIN_A0)){a=0;}
            while(1){
               if(input(PIN_A0)==1){a++;}
               else{goto next3;}
               delay_us(25);
            }
            next3:
            if(a>=30)r_data=(r_data>>1)+0x80;
            else r_data=(r_data>>1);
            i++;
         }
         noise3:
         if(i==8){ir_data1=r_data;r_data=0;}
         if(i==16){ir_data2=r_data;r_data=0;goto exit_rcv3;}
   }
   exit_rcv3:
   a = ir_data2;
   	switch(a){			                              // switch chek
	     	case 0x00:ir_rund=0x01;break;                //  1ch  left front turn
	     	case 0x08:ir_rund=0x05;break;                //  2ch  front
	     	case 0x10:ir_rund=0x04;break;                //  3ch  right front turn
	     	case 0x18:ir_rund=0x03;break;                //  4ch  left turn
	     	case 0x20:ir_rund=0x00;break;                //  5ch  stop
	     	case 0x28:ir_rund=0x06;break;                //  6ch  right turn
	     	case 0x30:ir_rund=0x08;break;                //  7ch  left back
	     	case 0x38:ir_rund=0x02;break;                //  8ch  back
	     	case 0x01:ir_rund=0x09;break;                //  9ch  right back
	     	case 0x09:ir_mode=0;ir_rund=0x00;break;      // 10ch  200msec mode
	     	case 0x11:ir_rund=0x0b;break;                // 11ch  cyoro cyoro
	     	case 0x19:ir_mode=1;ir_rund=0x00;break;      // 12ch  continue mode
   		default:goto ir_next3;break;
      }
   ir_run();
   goto ir_next3;
}

//  +++++++++++++++++  sharp mode  +++++++++++++++++
//       ch      ir_data1         ir_data2
//      1ch    1000 0000 0x80     ----0000
//      2ch    1000 1000 0x88     ----0000
//      3ch    1001 0000 0x90     ----0000
//      4ch    1001 1000 0x98     ----0000
//      5ch    1010 0000 0xa0     ----0000
//      6ch    1010 1000 0xa8     ----0000
//      7ch    1011 0000 0xb0     ----0000
//      8ch    1011 1000 0xb8     ----0000
//      9ch    1100 0000 0xc0     ----0000
//     10ch    1100 1000 0xc8     ----0000
//     11ch    1101 0000 0xd0     ----0000
//     12ch    1101 1000 0xd8     ----0000
//   ++++++++++++++++++++++++++++++++++++++++++++++++++++

ir_rcv_shp(){                        // ir_recive sharp function
   int a,i,j;
   int r_data;

   ir_next4:
   i=0;
   r_data=0;

   while(1){
         if(input(PIN_A0)==0){
            for(j=0;j<10;j++){delay_us(10);a=0;
               if(input(PIN_A0)!=0){goto noise4;}
            }
            while(~input(PIN_A0)){a=0;}
            while(1){
               if(input(PIN_A0)==1){a++;}
               else{goto next4;}
               delay_us(30);
            }
            next4:
            if(a>=25)r_data=(r_data>>1)+0x80;
            else r_data=(r_data>>1);
            i++;
         }
         noise4:
         if(i==8){ir_data1=r_data;r_data=0;}
         if(i==12){r_data>>=4;ir_data2=r_data;r_data=0;goto exit_rcv4;}
   }
   exit_rcv4:
   a = ir_data1 & 0xf8;
   	switch(a){			                              // switch chek
	     	case 0x80:ir_rund=0x01;break;                //  1ch  left front turn
	     	case 0x88:ir_rund=0x05;break;                //  2ch  front
	     	case 0x90:ir_rund=0x04;break;                //  3ch  right front turn
	     	case 0x98:ir_rund=0x03;break;                //  4ch  left turn
	     	case 0xa0:ir_rund=0x00;break;                //  5ch  stop
	     	case 0xa8:ir_rund=0x06;break;                //  6ch  right turn
	     	case 0xb0:ir_rund=0x08;break;                //  7ch  left back
	     	case 0xb8:ir_rund=0x02;break;                //  8ch  back
	     	case 0xc0:ir_rund=0x09;break;                //  9ch  right back
	     	case 0xc8:ir_mode=0;ir_rund=0x00;break;      // 10ch  200msec mode
	     	case 0xd0:ir_rund=0x0b;break;                // 11ch  cyoro cyoro
	     	case 0xd8:ir_mode=1;ir_rund=0x00;break;      // 12ch  continue mode
   		default:ir_rund=0x00;delay_us(2);break;
      }
   ir_run();
   goto ir_next4;
}

sony(){
   ir_rcv_son();     // sony ir function
}
victor(){
   ir_rcv_vic();     // victor ir function
}
mitubish(){
   ir_rcv_mit();     // mitubish ir function
}
sharp(){
   ir_rcv_shp();     // sharp ir function
}

mode3(){
   int key;

   led_moni_on = 0x01;               // LED4 ON

	enable_interrupts(INT_RTCC);		 // interrupt
	enable_interrupts(GLOBAL);		    // interrupt start OK
   delay_ms(2900);

   disable_interrupts(GLOBAL);      // interrupt stop
   led_moni_on = 0x00;              // LED OFF

   ir_mode=0;                        // ir_mode = 0 set

   while(1){
     key =~port_a & 0x3c;            // switch input

   	switch(key){			             // switch chek

   		case 0x20:                     // switch-0   **1000** (red)
	     		sony();    		             // sony mode
		    	break;
	     	case 0x10:                     // switch-1   **0100** (yellow)
		    	victor();    	             // victor mode
			   break;
   		case 0x08:                     // switch-2   **0010** (blue)
	     		mitubish();    	          // mitubish mode
		    	break;
   		case 0x04:                     // switch-3   **0001** (white)
	     		sharp();       	          // sharp mode
		    	break;

   		default:delay_ms(120);break;
      }

	}
}


/* ============== main function ============== */
main() {
   int mode;
	io_set();		      // i/o port setup
	interrupt_set();	   // interrupt setup
   int_count=61;        // prescale : 61

   delay_ms(200);       // 200msec waite

   mode =~port_a & 0xc0;   // DIP-SW check

	switch(mode){			   // mode check
		case 0x00:           // mode-0
			mode0();    		// goto mode-0
			break;            // demo program & sensor mode

		case 0x80:
			mode1();    		// goto mode-1
			break;            // teaching mode

		case 0x40:
			mode2();    		// goto mode-2
			break;            // rs232c resive mode

		case 0xc0:
			mode3();    		// goto mode-3
			break;            // infrared remote-control mode

		default:delay_ms(20);break;

	}
	while(1) { 	}           // endless LOOP
}
