/* ****************************************************** */
/*     jikkyo-robot system program  ver.2  pic16f88       */
/*                                                        */
/*     file name : system88_2.c                           */
/*                                                        */
/*     program is CCS-C PICC Compiler                     */
/*                  (PCB & PCM Version 3.183)             */
/*                                                        */
/*     Y/M/D  2011/8/28 programmed by jikyo company       */
/*                                                        */
/*     Ver.2  fW^suRɑΉ                  */
/* ****************************************************** */
// ************************** I/O *******************************
//  port_a -------- input port
//    PA0 ----- infra sensa RIGHT & RS232C & main infra sensa
//    PA1 ----- infra sensa LEFT
//    PA2 ----- SWICH-3(SW3) red switch
//    PA3 ----- SWICH-2(SW2) yellow switch
//    PA4 ----- SWICH-1(SW1) blue switch
//    PA5 ----- SWICH-0(SW0) white switch
//                                   mode0 mode1 mode2 mode3
//    PA6 ----- DIP-SWICH-0     PA6   1      1     0     0
//    PA7 ----- DIP-SWICH-1     PA7   1      0     1     0
//
// port_b ---------- output port
//                               LED            
//    PB0 ----- LED3                  LED3 LED2 LED1 LED0
//    PB1 ----- LED2                  PB0  PB1  PB2  PB3
//    PB2 ----- LED1
//    PB3 ----- LED0 & infra LED
//                                   stop    cw     ccw   stop
//    PB4 ----- MOTOR control RIGHT    0      1      0     1
//    PB5 ----- MOTOR control RIGHT    0      0      1     1
//    PB6 ----- MOTOR control LEFT     0      1      0     1
//    PB7 ----- MOTOR control LEFT     0      0      1     1
//
// ***************************************************************
#include <16f88.H>                     // include file set
#fuses INTRC_IO,NOWDT,PUT,NOPROTECT,NOLVP,NOMCLR,BROWNOUT
#fuses NOCPD,NOWRT,NODEBUG,CCPB0

#use Delay(Clock=4000000)	            // cpu speed 4MHz set (intosc)

#byte port_a=5		      // port_a address set bank0 05h
#byte port_b=6		      // port_b address set bank0 06h

#define int_per_sec 61	// 4000000/(4*256*64)=61.035
//                         interrupt cycle = 1.0*256*64=16.384msec

int int_count;		// interrupt times count
int infra;        // infra data
int qs_mode;      // Q-STEER mode=0 A or B   mode=1 C or D
int led_moni_on,led_moni_off;          // led monita on & off

//   ԍ     0    1    2    3    4    5    6    7      8    9
//           stop    E Oi E `  E
int move_led[10]={0x00,0x04,0x02,0x06,0x01,0x05,0x03,0x07,0x00,0x00};
int move_run[10]={0x00,0x10,0xa0,0x90,0x40,0x50,0x60,0xf0,0x20,0x80};

//   ********      *********
//   ԍ@@   PB7 PB6 PB5 PB4  run data  PB0  PB1  PB2  PB3  led data    3LED monitor
//      0      stop      0   0   0   0     0x00     0    0    0    -      0x00    @@
//      1            0   0   0   1     0x10     0    0    1    -      0x04    @@
//      2            1   0   1   0     0xa0     0    1    0    -      0x02    @@
//      3          1   0   0   1     0x90     0    1    1    -      0x06        
//      4      E      0   1   0   0     0x40     1    0    0    -      0x01        
//      5      Oi      0   1   0   1     0x50     1    0    1    -      0x05        
//      6      E    0   1   1   0     0x60     1    1    0    -      0x03        
//      7      `    1   1   1   1     0xf0     1    1    1    -      0x07        
//      8      ށ@@0   0   1   0     0x20     0    0    0    -      0x00    ~  ~  ~ not LED
//      9      Eށ@@1   0   0   0     0x80     0    0    0    -      0x00    ~  ~  ~ not LED
//   @``Ƃ́A܂ƉE܂0.3msecJԂđOi铮
//
//
//   Ԑݒ       0    1    2    3    4    5    6    7
int move_time[8]={0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07};
int times_led[8]={0x08,0x0c,0x0a,0x0e,0x09,0x0d,0x0b,0x0f};

//   ********      *********
//   Ԕԍ@@ԁ@   data      PB0  PB1  PB2  PB3   led data    LED
//      0      0   0  sec   0x00       0    0    0    1      0x08      @@  
//      1      1   0.5sec   0x01       0    0    1    1      0x0c      @@  
//      2      2   1  sec   0x02       0    1    0    1      0x0a      @@  
//      3      3   1.5sec   0x03       0    1    1    1      0x0e            
//      4      4   2  sec   0x04       1    0    0    1      0x09            
//      5      5   2.5sec   0x05       1    0    1    1      0x0d            
//      6      6   3  sec   0x06       1    1    0    1      0x0b            
//      7      7   3.5sec   0x07       1    1    1    1      0x0f            

//       eep-rom data seting dimension(MAX=64)
int ee_rom_data[64];

//       mode3 key data
int ir_data[6],ir_rund,ir_mode;

// eep-rom data  adress 0x2100
#rom 0x2100={0x07,0x06, 0x00,0x00, //  1 cyorocyoro 6time  ,  2 stop
             0x00,0x00, 0x00,0x00, //  3 stop              ,  4 stop
             0x00,0x00, 0x00,0x00, //  5 stop              ,  6 stop
             0x00,0x00, 0x00,0x00, //  7 stop              ,  8 stop
             0x00,0x00, 0x00,0x00, //  9 stop              ,  10 stop
             0x00,0x00, 0x00,0x00, // 11 stop              ,  12 stop
             0x00,0x00, 0x00,0x00, // 13 stop              ,  14 stop
             0x00,0x00, 0x00,0x00, // 15 stop              ,  16 stop
             0x00,0x00, 0x00,0x00, // 17 stop              ,  18 stop
             0x00,0x00, 0x00,0x00, // 19 stop              ,  20 stop
             0x00,0x00, 0x00,0x00, // 21 stop              ,  22 stop
             0x00,0x00, 0x00,0x00, // 23 stop              ,  24 stop
             0x00,0x00, 0x00,0x00, // 25 stop              ,  26 stop
             0x00,0x00, 0x00,0x00, // 27 stop              ,  28 stop
             0x00,0x00, 0x00,0x00, // 29 stop              ,  30 stop
             0x00,0x00, 0x00,0x00} // 31 stop              ,  32 stop

/* ++++++++++++++ interrupt set +++++++++++++++++ */
// +++++++++++++++++++++++++++++++++++++++++++++++++
//  Timer0 is setuped below.
//    clock : 4MHz then cycle : 1.0usec
//    prescale : 61
//      then interrupt cycle = 1.0*256*64=16.384msec
//      then interval = 16.384msec
//    Include the standard header file.
// +++++++++++++++++++++++++++++++++++++++++++++++++
interrupt_set(){
	setup_counters(RTCC_INTERNAL,RTCC_DIV_64); // RTCC_INTERNAL=CLOCK USE 1/64
}

/* +++++++++ RTCC interrupt function ++++++++++++++ */
#INT_RTCC			          // interrupt tmr0
rtcc_isr(){			          // This function is called every time
	if(--int_count==0){	  	 // the RTCC(timer0) overflows(255-->0)
		int_count=int_per_sec;// 61times/second ( 16.384*61=0.99942sec = about 1sec)
	}
   if(int_count>=30){port_b=led_moni_on;}
   else port_b=led_moni_off;
}

/* +++++++++++++++ i/o set function ++++++++++++++ */
io_set(){
	set_tris_a(0xff);         // port_a = input seting
	set_tris_b(0x00);         // port_b = output seting
   port_b=0x00;              // motor stop
   led_moni_off=0x00;        // LED monitor off
}

/* ********************** stop **************** */
stop_clr(){                  // stop
      port_b = 0x00;         // stop & led clr
}
stop_brake(){                // stop brake
      port_b = 0xf0;         // brake
      delay_ms(100);         // 100msec brake
      port_b = 0x00;         // stop & led clr
}

/* ********************** cyorocyoro **************** */
cyoro(int time){                          // ``si֍ssj
   int i;

   for(i=0;i<time;i++){
      port_b = move_led[1] | 0xd0;              //  ( brake   )
      delay_ms(200);                            //  200msec run
      port_b = move_led[4] | 0x70;              //  E( @brake )
      delay_ms(200);                            //  200msec run
   }
   stop_clr();             //  stop
}

/* ********************* eep-rpm function  ************************* */
clr_data(){                // eep_rom_data[0] to [63] clr
   int i;
   for(i=0;i<=64;i++){
      ee_rom_data[i]=0x00;
   }
}

eep_write(){             // eep_rom_data[i] ====> eeprom  (writing program)
   int i;
   long rom_adress;

   rom_adress = 0x2100;             // rom_adress first data
   for(i=0;i<64;i++){
      write_eeprom(rom_adress,ee_rom_data[i]);   //rom data write
      rom_adress++;
   }
}

eep_read(){              // eeprom =====> ee_rom_data[i] (reading program)
   int i;
   long rom_adress;

   rom_adress = 0x2100;             // rom_adress first data
   for(i=0;i<64;i++){
      ee_rom_data[i]=read_eeprom(rom_adress);   //rom data read
      rom_adress++;
   }
}

eep_err(){                // eep err led
   led_moni_on = 0x08;              // LED0 ON
	enable_interrupts(INT_RTCC);		// interrupt
	enable_interrupts(GLOBAL);		   // interrupt start OK
   delay_ms(1000);                  // waite 1sec
   while(1){
   }
}

eep_end(){                 // end set
   led_moni_on = 0x01;              // LED3 ON
	enable_interrupts(INT_RTCC);		// interrupt
	enable_interrupts(GLOBAL);		   // interrupt start OK
   delay_ms(3000);                  // waite 3sec
   led_moni_on = 0x00;              // LED OFF
   disable_interrupts(GLOBAL);      // interrupt stop
}

run_eep(){                 // eep_rom_program data reading run
   int i,j;
   int time;

   for(i=0;i<32;i++){
      port_b = move_run[ee_rom_data[2*i]] | move_led[ee_rom_data[2*i]]; // read move data & led
      time = ee_rom_data[2*i+1];                                        // read time data
      if(time==0){goto exit_run_eep;}                                      // end chek
      if(ee_rom_data[2*i] ==0x07){cyoro(time);}                         // if  cyoro mode run
      else {for(j=0;j<time;j++){delay_ms(500);}}                        // else 0.5ms*time
   }
   exit_run_eep:
   stop_clr();       // port_b clr
   eep_end();        // end display set ( 3sec LED ) & return
}

/* ****************** mode-0 function ****************** */
/* @@@fvOP@Q@@@                      */
/* @@@ZT䎩[h                            */
/* ***************************************************** */
// ---------- run1 = demo1 -----------
run1(){
  int i;
	disable_interrupts(GLOBAL);		       // interrupt stop
   port_b = move_led[5] | move_run[5];     //  Oi
   delay_ms(2000);                         //  waite 2sec
   port_b = move_led[2] | move_run[2];     //  
   delay_ms(2000);                         //  waite 2sec
   port_b = move_led[3] | move_run[3];     //  
   delay_ms(2000);                         //  waite 2sec
   port_b = move_led[6] | move_run[6];     //  E
   delay_ms(2000);                         //  waite 2sec
   cyoro(5);                               //  cyorocyoro  5times
   stop_clr();                             //  stop
	enable_interrupts(GLOBAL);		          //  interrupt start
}
// ---------- run2 = demo2 ----------
run2(){
  int i;
   disable_interrupts(GLOBAL);		       // interrupt stop
   for(i=1;i<7;i++){
      port_b = move_led[i] | move_run[i];  //  run data output
      delay_ms(1500);                      //  waite 1.5sec
      stop_brake();                        //  brake
      delay_ms(1500);                      //  waite 1.5sec
   }
   cyoro(3);                               //  cyorocyoro  3times
   stop_clr();                             //  stop
	enable_interrupts(GLOBAL);		          // interrupt start
}
// ---------- run3 = jiritu program ----------
inf_check(){                     // sensor check program
   int i,sen_r,sen_l,sen;

   sen=0;
   sen_r=0;sen_l=0;

   set_pwm1_duty(0x1A);          // 38KHz on => [infra =on ]

   for(i=0;i<10;i++){            // sensor check 10 count
      if(input(PIN_A0)==0){sen_r++;} // right sensor check 10count
      if(input(PIN_A1)==0){sen_l++;} // left sensor check 10count
      delay_us(100);             // 100usec * 10count = 1mse about
   }

   if(sen_r>=8){sen=0x01;}       // right sensor [1]
   else{sen=0x00;}               // right sensor [0]
   if(sen_l>=8){sen=sen+0x02;}   // left sensor [1]
   else{sen=sen;}                // left sensor [0]

   set_pwm1_duty(0);             // PWM off

   delay_ms(1);                  // waite 1msec

   return(sen);                  // sensor data return
                                 // 0x00  off off ;0x03 on on  ;0x02 on off  ;0x01 off on
}

run3(int jiritu){                // sensor run program
   int sen;
	disable_interrupts(GLOBAL);		       // interrupt stop

   setup_ccp1(CCP_PWM);                // CCP1 PWM MODE
   setup_timer_2(T2_DIV_BY_1,0x1A,1);  // 38kHz set
   set_pwm1_duty(0);                   // PWM off

   while(1){
      sen=inf_check();

	   switch(sen){			   // sensor check
		case 0x00:                              // sensor off
         port_b = move_led[5] | move_run[5];  //  Oi
         delay_ms(100);
			break;

		case 0x01:                              // sensor off on
			if(jiritu==1){
               port_b = move_led[3] | move_run[3]; //  
               delay_ms(1000);                 		//  1sec
         }
			else{
               port_b = move_led[2] | move_run[2]; //  
               delay_ms(1000);                 		//  1sec
               port_b = move_led[3] | move_run[3]; //  
               delay_ms(1000);                 		//  1sec
         }
			break;

		case 0x02:                              // sensor on off
			if(jiritu==1){
               port_b = move_led[6] | move_run[6]; //  E
               delay_ms(1000);                 		//  1sec
         }
			else{
               port_b = move_led[2] | move_run[2]; //  
               delay_ms(1000);                 		//  1sec
               port_b = move_led[6] | move_run[6]; //  E
               delay_ms(1000);                 		//  1sec
         }
			break;

		case 0x03:                              // sensor on on
			port_b = move_led[2] | move_run[2];       //  
         delay_ms(1000);                 		      //  1sec
        	break;

		default:delay_ms(2);break;
    }
   }
}

mode0(){                // mode-0 main program
   int key;
   int jiritu;

   led_moni_on = 0x08;              // LED0 ON
	enable_interrupts(INT_RTCC);		// interrupt
	enable_interrupts(GLOBAL);		   // interrupt start OK

   while(1){
      key =~port_a & 0x3c;
   	switch(key){			                       // switch chek
		   case 0x04:jiritu=1;run3(jiritu);break;   // switch-3 sensor run2
		   case 0x08:jiritu=0;run3(jiritu);break;   // switch-2 sensor run1
		   case 0x10:run2();break;                  // switch-1 run3 demo1
		   case 0x20:run1();break;                  // switch-0 run4 demo2
  		default:delay_ms(20);break;
      }
	}
}

/* ****************** mode-1 function ****************** */
/* @@@蓮͎s[h @                       */
/* @@@i{bg̊wK[hj                        */
/* ***************************************************** */
mode1(){
   int i,key;
   long rom_adress;
   int rom_data;
   int move_d;

   led_moni_on = 0x04;              // LED2 ON
	enable_interrupts(INT_RTCC);		// interrupt
	enable_interrupts(GLOBAL);		   // interrupt start OK
   delay_ms(2900);                  // waite 3-sec

   clr_data();                      //ee_rom_data <====== 0 set

   i = 0;                      // read adress first data
   rom_data = 0;               // rom_data =0

   disable_interrupts(GLOBAL);   // interrupt stop
   led_moni_on = 0x00;              // LED OFF
   stop_clr();

   while(1){
      loop:
      key =~port_a & 0x3c;                // key chek
      if(key==0){goto loop;}              // no-key ---> loop
      delay_ms(20);                       // chataring waite 20msec
         while((~port_a) & 0x3c){}        // pull key waite
     	switch(key){			               // key  chek
		   case 0x04:eep_read();            // key-4 eep_rom reading
                   i=0;rom_data=0;
                   break;
		   case 0x08:rom_data++;            // key-3 data seting( 0 to 7 )
                   if(rom_data==8){rom_data=0;};
                   ee_rom_data[i]=rom_data;
                   break;
		   case 0x10:i++;                   // key-2 data set and next adress
                   rom_data=0;
                   break;
		   case 0x20:eep_write();           // key-1 eep_rom write & run
                   run_eep();
                   i=0;
                   rom_data=0;
                   break;
  		   default:delay_ms(2);break;       // default
      }
      if(i==31){eep_write();eep_err();}   // i max 31 err

      if((i%2)==0){port_b = move_led[ee_rom_data[i]];}   // move data led set
      if((i%2)!=0){port_b =times_led[ee_rom_data[i]];}   // time data led set
      delay_ms(10);                                      // waite 10ms
   }
}

/* ****************** mode-2 function ****************** */
/* @@@qrQRQbʐM[h @                       */
/* @@@|PRVAʐM[h                      */
/* ***************************************************** */

/* ++++++++++++ rs232 function ++++++++++++ */
/*                                          */
/*     RS232C receive format                */
/*       baud rate 1200 bps                 */
/*       data bit     8 bit                 */
/*       stop bit     1 bit                 */
/*       parity        none                 */
/*       end of line     CR                 */
/*       end of file     1A                 */
/*                                          */
/*  +++++++++++++++++++++++++++++++++++++++ */
hex_ascii(int data){
   int a;

   if(data>=0x40){a=data-0x40+10;}
   else{a=data-0x30;}
   return(a);
}
/* ------ 1word get function (ASCII CODE)------ */
rs_1word(){
	int rs_data,i;
   rs_data=0;
   i=0;
   while(~input(PIN_A0)){}
   delay_us(1000);
   while(1){
     if(input(PIN_A0)==0){rs_data=(rs_data>>1)+0x80;}
     else {rs_data=(rs_data>>1);}
     delay_us(830);      // 1bit time = 1000000/1200=833usec
     i++;
     if(i==8){goto exit_rs1word;}
   }
   exit_rs1word:
   return(rs_data);
}
/* ------------- data get function ----------- */

// --------------- rs232c receive program ----------
rs_receive(){
	int data1,data2,i,j;
   clr_data();
   data1=rs_1word();
   j=0;
   while(1){                              // data receive
     data1=rs_1word();
     data2=rs_1word();
     ee_rom_data[j]=hex_ascii(data1);
     ee_rom_data[j+1]=hex_ascii(data2);
     if((data1==0x30)&(data2==0x30)){goto exit_rs;}
     j=j+2;
     if(j==32){goto exit_rs;}
	}
   exit_rs:
   eep_write();                           // eep rom data writing

   for(i=0;i<64;i=i+2){
     if((ee_rom_data[i]==0)&(ee_rom_data[i+1]==0)){goto exit_rs_end;}
     port_b=move_led[ee_rom_data[i]];
     delay_ms(2000);
     port_b=times_led[ee_rom_data[i+1]];
     delay_ms(2000);
   }
   exit_rs_end:
   port_b=0x00;
   return(0);
}
// --------------- pokekon receive program -------------
poke_receive(){
	int data1,data2,i,j;
   clr_data();
   for(i=0;i<9;i++){
      data1=rs_1word();
   }
   j=0;
   while(1){                              // data receive
     data1=rs_1word();
     data2=rs_1word();
     ee_rom_data[j]=hex_ascii(data1);
     ee_rom_data[j+1]=hex_ascii(data2);
     if((data1==0x30)&(data2==0x30)){goto exit_poke;}
     j=j+2;
     if(j==32){goto exit_poke;}
	}
   exit_poke:
   eep_write();                           // eep rom data writing

   for(i=0;i<64;i=i+2){                   // resiv data led set
     if((ee_rom_data[i]==0)&(ee_rom_data[i+1]==0)){goto exit_poke_end;}
     port_b=move_led[ee_rom_data[i]];
     delay_ms(2000);
     port_b=times_led[ee_rom_data[i+1]];
     delay_ms(2000);
   }
   exit_poke_end:
   port_b=0x00;
   return(0);

}
// ------ run4 = rs232c receive -----
run4(){
	disable_interrupts(GLOBAL);		       // interrupt stop
   rs_receive();                           // rs232c get data
	enable_interrupts(GLOBAL);		          // interrupt start
   led_moni_on = 0x02;                     // LED3 ON
}
// ------- run5 = pokekon receive -----
run5(){
	disable_interrupts(GLOBAL);		       // interrupt stop
   poke_receive();                         // pokekon sio get data
	enable_interrupts(GLOBAL);		          // interrupt start
   led_moni_on = 0x02;                     // LED3 ON
}

mode2(){
   int key;

   led_moni_on = 0x02;              // LED3 ON

	enable_interrupts(INT_RTCC);	  	 // interrupt RTCC
	enable_interrupts(GLOBAL);		    // interrupt start OK
   delay_ms(3000);                   // waite 3sec
	disable_interrupts(INT_RTCC);	  	 // interrupt stop RTCC

   while(1){
      key =~port_a & 0x3c;
   	switch(key){			           // switch check
		   case 0x04:eep_read();break;  // sw-3 eep_rom reading
		   case 0x08:run5();break;      // sw-2 run5 pokekon rs232c reciv & eep-rpm write
		   case 0x10:run4();break;      // sw-1 run4 rs232c reciv & eep-rom write
		   case 0x20:                   // sw-0 run
               	disable_interrupts(GLOBAL);		       // interrupt stop
                  run_eep();
               	enable_interrupts(GLOBAL);		          // interrupt start
                  led_moni_on = 0x02;                     // LED3 ON
                  break;
     		default:delay_ms(20);break;
      }
	}
}

/* ******************* mode-3 function ***************** */
/* @@@fW^erR䃂[h                */
/*           ( Ή[J[ )                            */
/* @@@@pi\jbNerR  red    switch   */
/* @@@@ŐerR          yellow switch   */
/* @@@@OHerR          blue   switch   */
/* @@@@V[verR      white + red     */
/* @@@@erR          white + yellow  */
/* @@@@rnmxerR      white + blue    */
/*    <>@̃vÓAׂẴRɑΉ   */
/*            Ă̂ł͂܂B               */
/* ***************************************************** */
ir_run(){
   if(ir_rund==0x0b){
      cyoro(1);
   }
   else{
      port_b = move_led[ir_rund] | move_run[ir_rund]; //  f[^o
      delay_ms(200);
   }
   if((ir_mode==1)&(ir_rund==0x00)){stop_brake();}    // ir_mode=1 stop ==>brake
   if(ir_mode==0){stop_clr();}                        // ir_mode=0 clr
}

//  +++++++++++++++++  sharp infrared code data  +++++++++++++++++++++++
//   @@iƓdntH[}bgj
//       ch      ir_data
//      1ch    55 5A F1 48 72 4C   :   55 5A F1 48 80 8D
//      2ch    55 5A F1 48 F2 44   :   55 5A F1 48 40 81
//      3ch    55 5A F1 48 0A 43   :   55 5A F1 48 C0 89
//      4ch    55 5A F1 48 8A 4B   :   55 5A F1 48 20 87
//      5ch    55 5A F1 48 4A 47   :   55 5A F1 48 A0 8F
//      6ch    55 5A F1 48 CA 4F   :   55 5A F1 48 60 83
//      7ch    55 5A F1 48 2A 41   :   55 5A F1 48 E0 8B
//      8ch    55 5A F1 48 AA 49   :   55 5A F1 48 10 84
//      9ch    55 5A F1 48 6A 45   :   55 5A F1 48 90 8C
//     10ch    55 5A F1 48 EA 4D   :   55 5A F1 48 50 80
//     11ch    55 5A F1 48 1A 42   :   55 5A F1 48 D0 88
//     12ch    55 5A F1 48 9A 4A   :   55 5A F1 48 30 86
//
//   ŏ̂SoCg  55 5A F1 48 sharp code(JX^R[hRQrbg)
//   㔼̂QoCg@** ** if[^R[hPUrbgj
//   f[^R[h͋@ɂĈقȂ̂łQ^CvɑΉ
//
//   @wb_[̃^C~O@@@O̐M@@@     P̐M
//    |__________|~~~~~~~|__   |______|~~~~~~|  |_______|~~~~~~~~~~~|
//        3.2ms    1.6ms        0.4ms   0.4ms     0.4ms    0.8ms
//   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

sharp(){                        // ir_receive sharp function
   int a,i,j;
   int r_data;

   while(1){
         noise_sharp:
         a=0;
         while(!input(PIN_A0)){delay_us(50);a++;}
         if(a<40){goto noise_sharp;}            // header chek (2msec)

         while(input(PIN_A0)){a=0;}             // waite header data end

         for(i=0;i<6;i++){                      // data get 6 byte
            for(j=0;j<8;j++){                   // data get 8 bit
               while(~input(PIN_A0)){a=0;}      // PIN_A0 LOW time waite

               while(1){                        // PIN_A0 high time count
                  if(input(PIN_A0)==1){a++;}
                  else{goto next_sharp;}
                  delay_us(20);
               }
               next_sharp:
               if(a>=25){r_data=(r_data<<1)+0x01;} // 20*25=0.5msec over   time ==> 1
               else{r_data=(r_data<<1);}           // 20*25=0.5msec under  time ==> 0
            }
            ir_data[i]=r_data;                    // data set 1 byte
            r_data=0;
          }

   	switch(ir_data[4]){			                   // TV switch chek

	     	case 0x72:ir_rund=0x01;break;              //  1ch  left front turn
	     	case 0xf2:ir_rund=0x05;break;              //  2ch  front
	     	case 0x0a:ir_rund=0x04;break;              //  3ch  right front turn
	     	case 0x8a:ir_rund=0x03;break;              //  4ch  left turn
	     	case 0x4a:ir_rund=0x00;break;              //  5ch  stop
	     	case 0xca:ir_rund=0x06;break;              //  6ch  right turn
	     	case 0x2a:ir_rund=0x08;break;              //  7ch  left back
	     	case 0xaa:ir_rund=0x02;break;              //  8ch  back
	     	case 0x6a:ir_rund=0x09;break;              //  9ch  right back
	     	case 0xea:ir_mode=0;ir_rund=0x00;break;    // 10ch  one time mode
	     	case 0x1a:ir_rund=0x0b;break;              // 11ch  cyoro cyoro
	     	case 0x9a:ir_mode=1;ir_rund=0x00;break;    // 12ch  continue mode

	     	case 0x80:ir_rund=0x01;break;              //  1ch  left front turn
	     	case 0x40:ir_rund=0x05;break;              //  2ch  front
	     	case 0xc0:ir_rund=0x04;break;              //  3ch  right front turn
	     	case 0x20:ir_rund=0x03;break;              //  4ch  left turn
	     	case 0xa0:ir_rund=0x00;break;              //  5ch  stop
	     	case 0x60:ir_rund=0x06;break;              //  6ch  right turn
	     	case 0xe0:ir_rund=0x08;break;              //  7ch  left back
	     	case 0x10:ir_rund=0x02;break;              //  8ch  back
	     	case 0x90:ir_rund=0x09;break;              //  9ch  right back
	     	case 0x50:ir_mode=0;ir_rund=0x00;break;    // 10ch  one time mode
	     	case 0xd0:ir_rund=0x0b;break;              // 11ch  cyoro cyoro
	     	case 0x30:ir_mode=1;ir_rund=0x00;break;    // 12ch  continue mode
   		default:ir_rund=0x00;delay_us(2);break;
      }
   ir_run();
   }
}

//  +++++++++++++++++  panasonic infrared code data  ++++++++++++++++++++
//   @@iƓdntH[}bgj
//       ch      ir_data
//      1ch    40 04 01 90 02 93   :   40 04 01 00 08 09
//      2ch    40 04 01 90 82 13   :   40 04 01 00 88 89
//      3ch    40 04 01 90 42 D3   :   40 04 01 00 48 49
//      4ch    40 04 01 90 C2 53   :   40 04 01 00 C8 C9
//      5ch    40 04 01 90 22 B3   :   40 04 01 00 28 29
//      6ch    40 04 01 90 A2 33   :   40 04 01 00 A8 A9
//      7ch    40 04 01 90 62 F3   :   40 04 01 00 68 69
//      8ch    40 04 01 90 E2 73   :   40 04 01 00 E8 E9
//      9ch    40 04 01 90 12 83   :   40 04 01 00 18 19
//     10ch    40 04 01 90 92 03   :   40 04 01 00 98 99
//     11ch    40 04 01 90 52 C3   :   40 04 01 00 58 59
//     12ch    40 04 01 90 D2 43   :   40 04 01 00 D8 D9
//
//   ŏ̂SoCg  40 04 01 ** panasonic code(JX^R[hRQrbg)
//   **9000͐R[ĥߖ
//   㔼̂QoCg@** ** if[^R[hPUrbgj
//   f[^R[h͋@ɂĈقȂ̂łQ^CvɑΉ
//   AŏISrbg̓}XNAQOrbgŎ
//
//   @wb_[̃^C~O@@@O̐M@@@     P̐M
//    |__________|~~~~~~~|__   |______|~~~~~~|  |_______|~~~~~~~~~~~|
//        3.2ms    1.6ms        0.4ms   0.4ms     0.4ms    0.8ms
//   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

pana(){                        // ir_receive panasonic function
   int a,i,j;
   int r_data;

   while(1){
         noise_pana:
         a=0;
         while(!input(PIN_A0)){delay_us(50);a++;}
         if(a<40){goto noise_pana;}           // header chek (2msec)

         while(input(PIN_A0)){a=0;}       // waite header data end

         for(i=0;i<6;i++){                // data get 6 byte
            for(j=0;j<8;j++){             // data get 8 bit
               while(~input(PIN_A0)){a=0;}   // PIN_A0 LOW time waite

               while(1){                     // PIN_A0 high time count
                  if(input(PIN_A0)==1){a++;}
                  else{goto next_pana;}
                  delay_us(20);
               }
               next_pana:
               if(a>=25){r_data=(r_data<<1)+0x01;} // 20*25=0.5msec over   time ==> 1
               else{r_data=(r_data<<1);}           // 20*25=0.5msec under  time ==> 0
            }
            ir_data[i]=r_data;                    // data set 1 byte
            r_data=0;
          }

   	switch(ir_data[4]){			                   // TV switch chek

	     	case 0x02:ir_rund=0x01;break;              //  1ch  left front turn
	     	case 0x82:ir_rund=0x05;break;              //  2ch  front
	     	case 0x42:ir_rund=0x04;break;              //  3ch  right front turn
	     	case 0xc2:ir_rund=0x03;break;              //  4ch  left turn
	     	case 0x22:ir_rund=0x00;break;              //  5ch  stop
	     	case 0xa2:ir_rund=0x06;break;              //  6ch  right turn
	     	case 0x62:ir_rund=0x08;break;              //  7ch  left back
	     	case 0xe2:ir_rund=0x02;break;              //  8ch  back
	     	case 0x12:ir_rund=0x09;break;              //  9ch  right back
	     	case 0x92:ir_mode=0;ir_rund=0x00;break;    // 10ch  one time mode
	     	case 0x52:ir_rund=0x0b;break;              // 11ch  cyoro cyoro
	     	case 0xd2:ir_mode=1;ir_rund=0x00;break;    // 12ch  continue mode

	     	case 0x08:ir_rund=0x01;break;              //  1ch  left front turn
	     	case 0x88:ir_rund=0x05;break;              //  2ch  front
	     	case 0x48:ir_rund=0x04;break;              //  3ch  right front turn
	     	case 0xc8:ir_rund=0x03;break;              //  4ch  left turn
	     	case 0x28:ir_rund=0x00;break;              //  5ch  stop
	     	case 0xa8:ir_rund=0x06;break;              //  6ch  right turn
	     	case 0x68:ir_rund=0x08;break;              //  7ch  left back
	     	case 0xe8:ir_rund=0x02;break;              //  8ch  back
	     	case 0x18:ir_rund=0x09;break;              //  9ch  right back
	     	case 0x98:ir_mode=0;ir_rund=0x00;break;    // 10ch  one time mode
	     	case 0x58:ir_rund=0x0b;break;              // 11ch  cyoro cyoro
	     	case 0xd8:ir_mode=1;ir_rund=0x00;break;    // 12ch  continue mode
   		default:ir_rund=0x00;delay_us(2);break;
      }
   ir_run();
   }
}

//  +++++++++++++++++  MITSUBISHI code ++++++++++++++++++++++++++++++
//   @@iOHIWitH[}bgj
//       ch   ir_data
//      1ch    E2 00
//      2ch    E2 10
//      3ch    E2 08
//      4ch    E2 18
//      5ch    E2 04
//      6ch    E2 14
//      7ch    E2 0C
//      8ch    E2 1C
//      9ch    E2 80
//     10ch    E2 90
//     11ch    E2 88
//     12ch    E2 98
//
//   OH́AАMQoCĝ
//   E2͐R[ĥߖ
//   㔼̂PoCgŎʂ
//
//   @wb_[̃^C~O@@O̐M@@@     P̐M
//    _|~~~~~~~~~~~|__      |_____|~~~~~~~|  |_____|~~~~~~~~~~~~~~|
//        5ms over            0.4ms  0.8ms    0.4ms    2ms
//   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

mitu(){                        // ir_receive mitubisi function
   int a,i,j;
   int r_data;

   while(1){
         noise_mitu:
         a=0;
         while(input(PIN_A0)==1){delay_us(50);a++;}
         if(a<100){goto noise_mitu;}           // header chek (5msec)

         while(input(PIN_A0)==1){a=0;}       // waite header data end

         for(i=0;i<2;i++){                // data get 2 byte
            for(j=0;j<8;j++){             // data get 8 bit
               while(~input(PIN_A0)){a=0;}   // PIN_A0 LOW time waite

               while(1){                     // PIN_A0 high time count
                  if(input(PIN_A0)==1){a++;}
                  else{goto next_mitu;}
                  delay_us(20);
               }
               next_mitu:
               if(a>=50){r_data=(r_data<<1)+0x01;} // 20*50=1msec over  time ==> 1
               else{r_data=(r_data<<1);}           // 20*50=1msec under time ==> 0
            }
            ir_data[i]=r_data;                    // data set 1 byte
            r_data=0;
          }
   	switch(ir_data[1]){			                   // TV switch chek

	     	case 0x00:ir_rund=0x01;break;              //  1ch  left front turn
	     	case 0x10:ir_rund=0x05;break;              //  2ch  front
	     	case 0x08:ir_rund=0x04;break;              //  3ch  right front turn
	     	case 0x18:ir_rund=0x03;break;              //  4ch  left turn
	     	case 0x04:ir_rund=0x00;break;              //  5ch  stop
	     	case 0x14:ir_rund=0x06;break;              //  6ch  right turn
	     	case 0x0c:ir_rund=0x08;break;              //  7ch  left back
	     	case 0x1c:ir_rund=0x02;break;              //  8ch  back
	     	case 0x80:ir_rund=0x09;break;              //  9ch  right back
	     	case 0x90:ir_mode=0;ir_rund=0x00;break;    // 10ch  one time mode
	     	case 0x88:ir_rund=0x0b;break;              // 11ch  cyoro cyoro
	     	case 0x98:ir_mode=1;ir_rund=0x00;break;    // 12ch  continue mode
    		default:ir_rund=0x00;delay_us(2);break;
      }
   ir_run();
   }
}

//  +++++++++++++++++ HITACHI infrared code data  ++++++++++++++++++
//   @@imdbntH[}bgj
//       ch      ir_data
//      1ch    6A 95 8E 71   :
//      2ch    6A 95 4E B1   :
//      3ch    6A 95 CE 31   :
//      4ch    6A 95 2E D1   :
//      5ch    6A 95 AE 51   :
//      6ch    6A 95 6E 91   :
//      7ch    6A 95 EE 11   :
//      8h     6A 95 1E E1   :
//      9ch    6A 95 9E 61   :
//     10ch    6A 95 0E F1   :
//     11ch    6A 95 5E A1   :
//     12ch    6A 95 DE 21   :
//
//   ŏ̂QoCg  6A 95  HITACHI code(JX^R[h16rbg)
//   㔼̂QoCg@** ** if[^R[hPUrbgj
//
//   @wb_[̃^C~O@@@O̐M@@@     P̐M
//    |__________|~~~~~~~|__   |______|~~~~~~|  |_______|~~~~~~~~~~~|
//      9ms        4.5ms        0.56ms 0.56ms     0.56ms    1.69ms
//   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

hita(){                        // ir_receive hitachi function
   int a,i,j;
   int r_data;

   while(1){
         noise_hita:
         a=0;
         while(!input(PIN_A0)){delay_us(100);a++;}
         if(a<80){goto noise_hita;}                // header chek  0 level (8ms over)

         a=0;
         while(input(PIN_A0)){delay_us(100);a++;}  // waite header data end
         if(a<35){goto noise_hita;}                // header chek  1 level (3.5ms over)

         for(i=0;i<4;i++){                         // data get 4 byte
            for(j=0;j<8;j++){                      // data get 8 bit
               while(~input(PIN_A0)){a=0;}         // PIN_A0 LOW time waite

               while(1){                           // PIN_A0 high time count
                  if(input(PIN_A0)==1){a++;}
                  else{goto next_hita;}
                  delay_us(20);
               }
               next_hita:
               if(a>=40){r_data=(r_data<<1)+0x01;} //  20*40=0.8msec over  time ==> 1
               else{r_data=(r_data<<1);}           //  20*40=0.8msec under time ==> 0
            }
            ir_data[i]=r_data;                     // data set 1 byte
            r_data=0;
          }
   	switch(ir_data[2]){			                   // TV switch chek

	     	case 0x8e:ir_rund=0x01;break;              //  1ch  left front turn
	     	case 0x4e:ir_rund=0x05;break;              //  2ch  front
	     	case 0xce:ir_rund=0x04;break;              //  3ch  right front turn
	     	case 0x2e:ir_rund=0x03;break;              //  4ch  left turn
	     	case 0xae:ir_rund=0x00;break;              //  5ch  stop
	     	case 0x6e:ir_rund=0x06;break;              //  6ch  right turn
	     	case 0xee:ir_rund=0x08;break;              //  7ch  left back
	     	case 0x1e:ir_rund=0x02;break;              //  8ch  back
	     	case 0x9e:ir_rund=0x09;break;              //  9ch  right back
	     	case 0x0e:ir_mode=0;ir_rund=0x00;break;    // 10ch  one time mode
	     	case 0x5e:ir_rund=0x0b;break;              // 11ch  cyoro cyoro
	     	case 0xde:ir_mode=1;ir_rund=0x00;break;    // 12ch  continue mode
    		default:ir_rund=0x00;delay_us(2);break;
      }
   ir_run();
   }
}

//  +++++++++++++++++ TOSHIBA infrared code data  ++++++++++++++++++
//   @@imdbntH[}bgj
//       ch      ir_data
//      1ch    02 FD 80 7F   :
//      2ch    02 FD 40 BF   :
//      3ch    02 FD C0 3F   :
//      4ch    02 FD 20 DF   :
//      5ch    02 FD A0 5F   :
//      6ch    02 FD 60 9F   :
//      7ch    02 FD E0 1F   :
//      8h     02 FD 10 EF   :
//      9ch    02 FD 90 6F   :
//     10ch    02 FD 50 AF   :
//     11ch    02 FD D0 2F   :
//     12ch    02 FD 30 CF   :
//
//   ŏ̂QoCg  02 FD  TOSIBA code(JX^R[h16rbg)
//   㔼̂QoCg@** ** if[^R[hPUrbgj
//
//   @wb_[̃^C~O@@@O̐M@@@     P̐M
//    |__________|~~~~~~~|__   |______|~~~~~~|  |_______|~~~~~~~~~~~|
//      9ms        4.5ms        0.56ms 0.56ms     0.56ms    1.69ms
//   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

tosi(){                        // ir_receive hitachi function
   int a,i,j;
   int r_data;

   while(1){
         noise_tosi:
         a=0;
         while(!input(PIN_A0)){delay_us(100);a++;}
         if(a<80){goto noise_tosi;}                // header chek  0 level (8ms over)

         a=0;
         while(input(PIN_A0)){delay_us(100);a++;}  // waite header data end
         if(a<35){goto noise_tosi;}                // header chek  1 level (3.5ms over)

         for(i=0;i<4;i++){                         // data get 4 byte
            for(j=0;j<8;j++){                      // data get 8 bit
               while(~input(PIN_A0)){a=0;}         // PIN_A0 LOW time waite

               while(1){                           // PIN_A0 high time count
                  if(input(PIN_A0)==1){a++;}
                  else{goto next_tosi;}
                  delay_us(20);
               }
               next_tosi:
               if(a>=40){r_data=(r_data<<1)+0x01;} //  20*40=0.8msec over  time ==> 1
               else{r_data=(r_data<<1);}           //  20*40=0.8msec under time ==> 0
            }
            ir_data[i]=r_data;                     // data set 1 byte
            r_data=0;
          }
   	switch(ir_data[2]){			                   // TV switch chek

	     	case 0x80:ir_rund=0x01;break;              //  1ch  left front turn
	     	case 0x40:ir_rund=0x05;break;              //  2ch  front
	     	case 0xc0:ir_rund=0x04;break;              //  3ch  right front turn
	     	case 0x20:ir_rund=0x03;break;              //  4ch  left turn
	     	case 0xa0:ir_rund=0x00;break;              //  5ch  stop
	     	case 0x60:ir_rund=0x06;break;              //  6ch  right turn
	     	case 0xe0:ir_rund=0x08;break;              //  7ch  left back
	     	case 0x10:ir_rund=0x02;break;              //  8ch  back
	     	case 0x90:ir_rund=0x09;break;              //  9ch  right back
	     	case 0x50:ir_mode=0;ir_rund=0x00;break;    // 10ch  one time mode
	     	case 0xd0:ir_rund=0x0b;break;              // 11ch  cyoro cyoro
	     	case 0x30:ir_mode=1;ir_rund=0x00;break;    // 12ch  continue mode
    		default:ir_rund=0x00;delay_us(2);break;
      }
   ir_run();
   }
}

//  +++++++++++++++++ sony infrared code data  ++++++++++++++++++
//   @@irnmxIWitH[}bgj
//       ch      ir_data
//      1ch    01 08
//      2ch    81 0C
//      3ch    41 0A
//      4ch    C1 0E
//      5ch    21 09
//      6ch    A1 0D
//      7ch    61 0B
//      8ch    E1 0F
//      9ch    11 08
//     10ch    91 0C
//     11ch    51 0A
//     12ch    D1 0E
//   f[^̂QoCg@** ** if[^R[hPUrbgj
//
//   @wb_[̃^C~O@@@O̐M@@@     P̐M
//    |__________|~~~~~~~|__   |______|~~~~~~|  |__________|~~~~~~|
//        2.4ms     0.6ms        0.4ms  0.4ms       0.8ms    0.4ms
//   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

sony(){                        // ir_receive sony function
   int a,i,j;
   int r_data;

   while(1){
         noise_sony:
         a=0;
         while(!input(PIN_A0)){delay_us(40);a++;}
         if(a<50){goto noise_sony;}              // header chek (2msec)

         while(input(PIN_A0)){a=0;}              // waite header data end

         for(i=0;i<2;i++){                       // data get 2 byte
            for(j=0;j<8;j++){                    // data get 8 bit
               a=0;
               while(1){
                  if(~input(PIN_A0)){a++;}       // PIN_A0 LOW time count
                  else{goto exit1;}
                  delay_us(20);
               }

               exit1:
               while(input(PIN_A0)){}              // PIN_A0 high time waite

               if(a>=30){r_data=(r_data<<1)+0x01;} // 20*30=0.6msec over  time ==> 1
               else{r_data=(r_data<<1);}           // 20*30=0.6msec under time ==> 0

            }
            ir_data[i]=r_data;                    // data set 1 byte
            r_data=0;
          }
   	switch(ir_data[0]){			                   // TV switch chek

	     	case 0x01:ir_rund=0x01;break;              //  1ch  left front turn
	     	case 0x81:ir_rund=0x05;break;              //  2ch  front
	     	case 0x41:ir_rund=0x04;break;              //  3ch  right front turn
	     	case 0xc1:ir_rund=0x03;break;              //  4ch  left turn
	     	case 0x21:ir_rund=0x00;break;              //  5ch  stop
	     	case 0xa1:ir_rund=0x06;break;              //  6ch  right turn
	     	case 0x61:ir_rund=0x08;break;              //  7ch  left back
	     	case 0xe1:ir_rund=0x02;break;              //  8ch  back
	     	case 0x11:ir_rund=0x09;break;              //  9ch  right back
	     	case 0x91:ir_mode=0;ir_rund=0x00;break;    // 10ch  one time mode
	     	case 0x51:ir_rund=0x0b;break;              // 11ch  cyoro cyoro
	     	case 0xd1:ir_mode=1;ir_rund=0x00;break;    // 12ch  continue mode
    		default:ir_rund=0x00;delay_us(2);break;
      }
   ir_run();

   }
}

// *****************************************************************************
mode3(){
   int key;

   led_moni_on = 0x01;               // LED4 ON

	enable_interrupts(INT_RTCC);		 // interrupt
	enable_interrupts(GLOBAL);		    // interrupt start OK
   delay_ms(2900);

   disable_interrupts(GLOBAL);      // interrupt stop
   led_moni_on = 0x00;              // LED OFF

   ir_mode=0;                        // ir_mode = 0 set

   while(1){
     key =~port_a & 0x3c;            // switch input

   	switch(key){			             // switch chek
   		case 0x20:                     // switch-0   **1000** (red)
	     		pana();    		             // panasonic mode
		    	break;
	     	case 0x10:                     // switch-1   **0100** (yellow)
		    	tosi();     	             // toshiba mode
			   break;
   		case 0x08:                     // switch-2   **0010** (blue)
	     		mitu();          	          // mitubishi mode
		    	break;
   		case 0x24:                     // switch-4 + switch-0   **1001** ( white + red )
	     		sharp();       	          // sharp mode
		    	break;
   		case 0x14:                     // switch-4 + switch-1   **0101** ( white + yellow )
	     		hita();    		             // hitachi mode
		    	break;
	     	case 0x0c:                     // switch-4 + switch-3   **0011** ( white + blue )
		    	sony();     	             // sony mode
			   break;

   		default:delay_ms(120);break;
      }

	}
}


/* ============== main function ============== */
main() {
   int mode;
	io_set();		      // i/o port setup
	interrupt_set();	   // interrupt setup
   int_count=61;        // prescale : 61

   delay_ms(200);       // 200msec waite

   mode =~port_a & 0xc0;   // DIP-SW check

	switch(mode){			   // mode check
		case 0x00:           // mode-0
			mode0();    		// goto mode-0
			break;            // demo program & sensor mode

		case 0x80:
			mode1();    		// goto mode-1
			break;            // teaching mode

		case 0x40:
			mode2();    		// goto mode-2
			break;            // rs232c resive mode

		case 0xc0:
			mode3();    		// goto mode-3
			break;            // infrared remote-control mode

		default:delay_ms(20);break;

	}
	while(1) { 	}           // endless LOOP
}
